@@ -77,6 +77,7 @@ namespace Beagle.Filters {
 		}
 
 		bool setup_done = false;
+		string archive_type = "zip";
 
 		private void SetupArchiveStream ()
 		{
@@ -87,32 +88,38 @@ namespace Beagle.Filters {
 			case "application/zip":
 				archive_stream = new ZipInputStream (Stream);
 				get_next_entry = GetNextEntryZip;
+				archive_type = "zip";
 				break;
 
 			case "application/x-bzip-compressed-tar":
 				archive_stream = new TarInputStream (new BZip2InputStream (Stream));
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar-bz2";
 				break;
 
 			case "application/x-compressed-tar":
 			case "application/x-tgz":
 				archive_stream = new TarInputStream (new GZipInputStream (Stream));
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar-gz";
 				break;
 
 			case "application/x-tar":
 				archive_stream = new TarInputStream (Stream);
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar";
 				break;
 
 			case "application/x-gzip":
 				archive_stream = new GZipInputStream (Stream);
 				get_next_entry = GetNextEntrySingle;
+				archive_type = "gzip";
 				break;
 
 			case "application/x-bzip":
 				archive_stream = new BZip2InputStream (Stream);
 				get_next_entry = GetNextEntrySingle;
+				archive_type = "bz2";
 				break;
 
 			default:
@@ -164,15 +171,10 @@ namespace Beagle.Filters {
 			++count;
 			total_size += a_entry.Size;
 
-			// FIXME: For nested archives, create uid:foo#bar
-			// instead of uid:foo#xxx#bar (avoid duplicates ?)
-			// FIXME: Construct correct Uri
-			// 1. Use Uri (foo, true): foo is not escaped. This creates serialization error for uris containing non-ascii character
-			// 2: Using Uri (foo): foo is escaped. This causes foo.zip#b#c (file c in archive b in archive foo) to be escaped to foo.zip#b%23c. This should be properly escaped back. Note that, file b#c in archive foo.zip would also be escaped to foo.zip#b%23c. If there is any problem about incorrect filename containing '#' in an archive, then System.Uri should be abandoned and a custom Beagle.Uri should be created which can handle multiple fragments.
-			// If you read the Uri related FIXMEs and hacks in beagled/*Queryable,
-			// then you would see why creating Beagle.Uri class over System.Uri
-			// is not at all a bad idea :)
-			child = new Indexable (new Uri (Indexable.Uri.ToString () + "#" + a_entry.Name));
+			// Add "#<escaped-path-to-entry>" to the end of the Indexable Uri
+			// So, file b#c in archive foo.zip becomes file:///foo.zip#b%23c
+			// And file c in archive b in archive foo.zip becomes file:///foo.zip#b#c
+			child = new Indexable (UriFu.AddFragment (Indexable.Uri, a_entry.Name, false));
 
 			child.CacheContent = true;
 			child.MimeType = a_entry.MimeType;
@@ -181,7 +183,11 @@ namespace Beagle.Filters {
 			child.ContentUri = UriFu.PathToFileUri (a_entry.TempFile);
 			child.DeleteContent = true;
 
+			// FIXME Remove fixme:inside_archive during Property Hack Week
+			// Replace most flag properties by value properties
 			child.AddProperty (Property.NewBool ("fixme:inside_archive", true));
+			// Use this instead of fixme:inside_archive
+			child.AddProperty (Property.NewKeyword ("archive:type", archive_type));
 
 			child.AddProperty (Property.NewKeyword ("fixme:relativeuri", a_entry.Name));
 			child.AddProperty (Property.New ("fixme:comment", a_entry.Comment));
@@ -221,12 +227,17 @@ namespace Beagle.Filters {
 				return null;
 
 			string filename = FileSystem.GetTempFileName (extension);
-			FileStream file_stream = File.OpenWrite (filename);
+			FileStream file_stream = new FileStream (filename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite); // FileShare.ReadWrite needed for setting the mtime
+
+			// When we dump the contents of an indexable into a file, we
+			// expect to use it again soon.
+			FileAdvise.PreLoad (file_stream);
 
 			//Log.Debug ("Storing archive contents in {0}", filename);
-			
-			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 384); // 384 is 0600
-			
+
+			File.SetLastWriteTimeUtc (filename, mtime); // change this before making read-only
+			Mono.Unix.Native.Syscall.chmod (filename, Mono.Unix.Native.FilePermissions.S_IRUSR);
+
 			BufferedStream buffered_stream = new BufferedStream (file_stream);
 
 			byte [] buffer = new byte [8192];
@@ -280,14 +291,10 @@ namespace Beagle.Filters {
 			buffered_stream.Close ();
 
 			if (skip_file) {
-				File.Delete (filename);
+				FileSystem.PosixDelete (filename);
 				return null;
 			}
 
-			File.SetLastWriteTimeUtc (filename, mtime);
-
-			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256); // 256 is 0400
-
 			return filename;
 		}
 
