@@ -11,6 +11,7 @@ namespace SemWeb {
 		// TODO: Make some of the errors warnings.
 	
 		XmlReader xml;
+		bool closed;
 		
 		Hashtable blankNodes = new Hashtable();
 		UriMap namedNodes = new UriMap();
@@ -28,25 +29,80 @@ namespace SemWeb {
 			rdfObject = "http://www.w3.org/1999/02/22-rdf-syntax-ns#object",
 			rdfStatement = "http://www.w3.org/1999/02/22-rdf-syntax-ns#Statement";
 		
+		#if !SILVERLIGHT
 		public RdfXmlReader(XmlDocument document) {
 			xml = new XmlBaseAwareReader(new XmlNodeReader(document));
+			LoadNamespaces();
 		}
+		#endif
 		
 		public RdfXmlReader(XmlReader document) {
-			XmlValidatingReader reader = new XmlValidatingReader(document);
-			reader.ValidationType = ValidationType.None;
+			#if !DOTNET2
+				XmlValidatingReader reader = new XmlValidatingReader(document); // decodes entity definitions
+				reader.ValidationType = ValidationType.None;
+			#elif SILVERLIGHT
+				XmlReader reader = document;
+			#else
+				XmlReaderSettings settings = new XmlReaderSettings();
+				settings.ValidationType = ValidationType.None;
+				settings.ProhibitDtd = false;
+				settings.CloseInput = true;
+				XmlReader reader = XmlReader.Create(document, settings);
+			#endif
+        
 			xml = new XmlBaseAwareReader(reader);
+			LoadNamespaces();
 		}
 		
+		#if !SILVERLIGHT
 		public RdfXmlReader(TextReader document) : this(new XmlTextReader(document)) {
 		}
+		#else
+		public RdfXmlReader(TextReader document) {
+			throw new NotSupportedException("Reading RDF/XML from a TextReader is not supported in the Silverlight build of the SemWeb library. Someone needs to fix this.");
+		}
+		#endif
 
-		public RdfXmlReader(Stream document) : this(new XmlTextReader(document)) {
+		public RdfXmlReader(Stream document) : this(new StreamReader(document)) {
 		}
 
-		public RdfXmlReader(string file) : this(GetReader(file)) {
+		public RdfXmlReader(TextReader document, string baseUri) : this(document) {
+			BaseUri = baseUri;
+		}
+
+		public RdfXmlReader(Stream document, string baseUri) : this(new StreamReader(document), baseUri) {
+		}
+
+		public RdfXmlReader(string file, string baseUri) : this(GetReader(file), baseUri) {
+		}
+
+		public RdfXmlReader(string file) : this(GetReader(file), "file:///" + file) {
 		}
 		
+		protected override void Dispose() {
+			if (!closed)
+				xml.Close();
+			closed = true;
+		}
+
+		private void LoadNamespaces() {
+			// Move to the document element and load any namespace
+			// declarations on the node.
+
+			while (xml.Read()) {
+				if (xml.NodeType != XmlNodeType.Element) continue;
+
+				if (xml.MoveToFirstAttribute()) {
+					do {
+						if (xml.Prefix == "xmlns")
+							Namespaces.AddNamespace(xml.Value, xml.LocalName);
+					} while (xml.MoveToNextAttribute());
+					xml.MoveToElement();
+				}
+				break;
+			}
+		}
+
 		public override void Select(StatementSink storage) {
 			// Read past the processing instructions to
 			// the document element.  If it is rdf:RDF,
@@ -55,8 +111,14 @@ namespace SemWeb {
 			// description.
 			
 			this.storage = storage;
-									
-			while (xml.Read()) {
+
+			bool first = true; // on the first iteration don't
+											   // advance to the next node -- we already did that
+			while (first || xml.Read()) {
+				first = false;
+
+				if (xml.NodeType != XmlNodeType.Element) continue;
+				
 				if (xml.NamespaceURI == NS.RDF && xml.LocalName == "RDF" ) {
 					// If there is an xml:base here, set BaseUri so
 					// the application can recover it.  It doesn't
@@ -69,11 +131,15 @@ namespace SemWeb {
 						if (xml.NodeType == XmlNodeType.Element)
 							ParseDescription();
 					}
-					break;
+					
+				} else {
+					ParseDescription();
+				
 				}
+				break;
 			}
-
-			xml.Close();
+			
+			Dispose();
 		}
 		
 		private string CurNode() {
@@ -87,7 +153,14 @@ namespace SemWeb {
 			if (xml.NamespaceURI == "" && BaseUri == null)
 				return "#" + xml.LocalName;
 
-			return xml.NamespaceURI + xml.LocalName;
+			return CheckUri(xml.NamespaceURI + xml.LocalName);
+		}
+		
+		private string CheckUri(string uri) {
+			string error = Entity.ValidateUri(uri);
+			if (error != null)
+				OnWarning("The URI <" + uri + "> is not valid: " + error);
+			return uri;
 		}
 		
 		private int isset(string attribute) {
@@ -95,7 +168,7 @@ namespace SemWeb {
 		}
 		
 		private string Unrelativize(string uri) {
-			return GetAbsoluteUri(xml.BaseURI != "" ? xml.BaseURI : BaseUri, uri);
+			return CheckUri(GetAbsoluteUri(xml.BaseURI != "" ? xml.BaseURI : BaseUri, uri));
 		}
 		
 		private Entity GetBlankNode(string nodeID) {
@@ -127,8 +200,6 @@ namespace SemWeb {
 			
 			string nodeID = xml.GetAttribute("nodeID", NS.RDF);
 			string about = xml.GetAttribute("about", NS.RDF);
-			//if (about == null)
-			//	about = xml.GetAttribute("about");
 			string ID = xml.GetAttribute("ID", NS.RDF);
 			if (isset(nodeID) + isset(about) + isset(ID) > 1)
 				OnError("An entity description cannot specify more than one of rdf:nodeID, rdf:about, and rdf:ID");
@@ -155,11 +226,12 @@ namespace SemWeb {
 			
 			// If the name of the element is not rdf:Description,
 			// then the name gives its type.
-			if (CurNode() != NS.RDF + "Description") {
-				if (IsRestrictedName(CurNode()) || IsDeprecatedName(CurNode()))
+			string curnode = CurNode();
+			if (curnode != NS.RDF + "Description") {
+				if (IsRestrictedName(curnode) || IsDeprecatedName(curnode))
 					OnError(xml.Name + " cannot be the type of a resource.");
-				if (CurNode() == NS.RDF + "li") OnError("rdf:li cannot be the type of a resource");
-				storage.Add(new Statement(entity, rdfType, (Entity)CurNode(), Meta));
+				if (curnode == NS.RDF + "li") OnError("rdf:li cannot be the type of a resource");
+				storage.Add(new Statement(entity, rdfType, (Entity)curnode, Meta));
 			}
 			
 			ParsePropertyAttributes(entity);
@@ -271,9 +343,6 @@ namespace SemWeb {
 
 			string ID = xml.GetAttribute("ID", NS.RDF);
 
-			Console.WriteLine ("({0})({1})({2})({3})({4})({5})({6})",
-				nodeID, resource, parseType, datatype, lang, predicate, ID);
-
 			if (nodeID != null && !IsValidXmlName(nodeID))
 				OnWarning("'" + nodeID + "' is not a valid XML Name");
 			if (ID != null && !IsValidXmlName(ID))
@@ -306,8 +375,10 @@ namespace SemWeb {
 				}
 			
 			} else if (parseType != null && parseType == "Literal") {
-				if (datatype == null)
-					datatype = "http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral";
+				if (datatype != null)
+					OnError("The attribute rdf:datatype is not valid on a predicate whose parseType is Literal.");
+
+				datatype = "http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral";
 				
 				if (ParsePropertyAttributes(new BNode()))
 					OnError("Property attributes are not valid when parseType is Literal");
@@ -367,7 +438,11 @@ namespace SemWeb {
 				if (xml.IsEmptyElement) {
 					objct = new Literal("", null, datatype);
 				} else {
+					#if !DOTNET2
 					objct = new Literal(xml.ReadString(), null, datatype);
+					#else
+					objct = new Literal(xml.ReadElementContentAsString(), null, datatype);
+					#endif
 					if (xml.NodeType != XmlNodeType.EndElement)
 						OnError("XML markup may not appear in a datatyped literal property.");
 				}
@@ -549,26 +624,27 @@ namespace SemWeb {
 			public override bool MoveToElement () { return _reader.MoveToElement(); }
 			public override bool MoveToFirstAttribute () { return _reader.MoveToFirstAttribute(); }
 			public override bool MoveToNextAttribute () { return _reader.MoveToNextAttribute(); }
-			public override bool ReadAttributeValue () { return _reader.ReadAttributeValue(); }
-			public override string ReadElementString () { return _reader.ReadElementString(); }
-			public override string ReadElementString (string name) { return _reader.ReadElementString(name); }
-			public override string ReadElementString (string localName, string namespaceName) { return _reader.ReadElementString(localName, namespaceName); }
 			public override void ReadEndElement () { _reader.ReadEndElement(); }
 			public override string ReadInnerXml () { return _reader.ReadInnerXml(); }
 			public override string ReadOuterXml () { return _reader.ReadOuterXml(); }
 			public override void ReadStartElement () { _reader.ReadStartElement(); }
 			public override void ReadStartElement (string name) { _reader.ReadStartElement(name); }
 			public override void ReadStartElement (string localName, string namespaceName) { _reader.ReadStartElement(localName, namespaceName); }
+			public override void Skip () { _reader.Skip(); }
+
+			#if !SILVERLIGHT
+			public override bool ReadAttributeValue () { return _reader.ReadAttributeValue(); }
+			public override string ReadElementString () { return _reader.ReadElementString(); }
+			public override string ReadElementString (string name) { return _reader.ReadElementString(name); }
+			public override string ReadElementString (string localName, string namespaceName) { return _reader.ReadElementString(localName, namespaceName); }
 			public override string ReadString () { return _reader.ReadString(); }
 			public override void ResolveEntity () { _reader.ResolveEntity(); }
-			public override void Skip () { _reader.Skip(); }
+			#endif
 			
 			public override int AttributeCount { get { return _reader.AttributeCount; } }
-			public override bool CanResolveEntity { get { return _reader.CanResolveEntity; } }
 			public override int Depth { get { return _reader.Depth; } }
 			public override bool EOF { get { return _reader.EOF; } }
 			public override bool HasAttributes { get { return _reader.HasAttributes; } }
-			public override bool HasValue { get { return _reader.HasValue; } }
 			public override bool IsDefault { get { return _reader.IsDefault; } }
 			public override bool IsEmptyElement { get { return _reader.IsEmptyElement; } }
 			public override string this [int i] { get { return _reader[i]; } }
@@ -580,11 +656,16 @@ namespace SemWeb {
 			public override XmlNameTable NameTable { get { return _reader.NameTable; } }
 			public override XmlNodeType NodeType { get { return _reader.NodeType; } }
 			public override string Prefix { get { return _reader.Prefix; } }
-			public override char QuoteChar { get { return _reader.QuoteChar; } }
 			public override ReadState ReadState { get { return _reader.ReadState; } }
 			public override string Value { get { return _reader.Value; } }
 			public override string XmlLang { get { return _reader.XmlLang; } }
 			public override XmlSpace XmlSpace { get { return _reader.XmlSpace; } }
+			
+			#if !SILVERLIGHT
+			public override bool CanResolveEntity { get { return _reader.CanResolveEntity; } }
+			public override bool HasValue { get { return _reader.HasValue; } }
+			public override char QuoteChar { get { return _reader.QuoteChar; } }
+			#endif
 		}		
 	}
 }
