@@ -29,23 +29,36 @@ using System.Collections;
 using System.IO;
 using System.Threading;
 
-using Mono.Data.SqliteClient;
+using Mono.Data.Sqlite;
 
 using Beagle.Util;
 
 namespace Beagle.Daemon {
 
+	// FIXME: This class isn't multithread safe!  This class does not
+	// ensure that different threads don't utilize a transaction started
+	// in a certain thread at the same time.  However, since all the
+	// writes to this database are handled by a single thread, this isn't
+	// currently a problem, but it would have to be fixed if we wanted to
+	// do this in the future.
+	//
+	// See http://bugzilla.gnome.org/show_bug.cgi?329022
+
 	public class FileAttributesStore_Sqlite : IFileAttributesStore {
 
 		// Version history:
 		// 1: Original version
 		// 2: Replaced LastIndexedTime with LastAttrTime
-		const int VERSION = 2;
+		// 3: Changed STRING to TEXT
+		// 4: Use (directory, filename) as unique constraint
+		const int VERSION = 4;
 
 		private SqliteConnection connection;
 		private BitArray path_flags;
 		private int transaction_count = 0;
-
+		public SqliteCommand ReadCommand;
+		public SqliteCommand InsertCommand;
+		public SqliteCommand DeleteCommand;
 		enum TransactionState {
 			None,
 			Requested,
@@ -120,28 +133,24 @@ namespace Beagle.Daemon {
 				SqliteUtils.DoNonQuery (connection,
 							"CREATE TABLE db_info (             " +
 							"  version       INTEGER NOT NULL,  " +
-							"  fingerprint   STRING NOT NULL    " +
+							"  fingerprint   TEXT NOT NULL    " +
 							")");
 
 				SqliteUtils.DoNonQuery (connection,
-							"INSERT INTO db_info (version, fingerprint) VALUES ({0}, '{1}')",
-							VERSION, index_fingerprint);
+							"INSERT INTO db_info (version, fingerprint) VALUES (@version, @index_fingerprint)",
+							new string [] {"@version", "@index_fingerprint"},
+							new object [] {VERSION, index_fingerprint});
 
 				SqliteUtils.DoNonQuery (connection,
-							"CREATE TABLE file_attributes (           " +
-							"  unique_id      STRING UNIQUE,          " +
-							"  directory      STRING NOT NULL,        " +
-							"  filename       STRING NOT NULL,        " +
-							"  last_mtime     STRING NOT NULL,        " +
-							"  last_attrtime  STRING NOT NULL,        " +
-							"  filter_name    STRING NOT NULL,        " +
-							"  filter_version STRING NOT NULL         " +
-							")");
-
-				SqliteUtils.DoNonQuery (connection,
-							"CREATE INDEX file_path on file_attributes (" +
-							"  directory,      " +
-							"  filename        " +
+							"CREATE TABLE file_attributes (         " +
+							"  unique_id      TEXT NOT NULL,        " +
+							"  directory      TEXT NOT NULL,        " +
+							"  filename       TEXT NOT NULL,        " +
+							"  last_mtime     TEXT NOT NULL,        " +
+							"  last_attrtime  TEXT NOT NULL,        " +
+							"  filter_name    TEXT NOT NULL,        " +
+							"  filter_version TEXT NOT NULL,        " +
+							"  UNIQUE (directory, filename)         " +
 							")");
 			} else {
 				SqliteCommand command;
@@ -174,6 +183,24 @@ namespace Beagle.Daemon {
 				Logger.Log.Debug ("Loaded {0} records from {1} in {2:0.000}s", 
 						 count, GetDbPath (directory), (dt2 - dt1).TotalSeconds);
 			}
+			ReadCommand = new SqliteCommand (this.connection);
+			ReadCommand.CommandText = "SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
+				"FROM file_attributes WHERE directory=@dir AND filename=@fname";
+			InsertCommand = new SqliteCommand (this.connection);
+			InsertCommand.CommandText = "INSERT OR REPLACE INTO file_attributes " +
+							" (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
+							" VALUES (@unique_id, @directory, @filename, @last_mtime, @last_attrtime, @filter_name, @filter_version)";
+			DeleteCommand = new SqliteCommand (this.connection);
+			DeleteCommand.CommandText = "DELETE FROM file_attributes WHERE directory=@directory AND filename=@filename";
+		}
+
+		public void Dispose ()
+		{
+			ReadCommand.Dispose ();
+			InsertCommand.Dispose ();
+			DeleteCommand.Dispose ();
+			connection.Dispose ();
+			connection = null;
 		}
 
 		///////////////////////////////////////////////////////////////////
@@ -187,8 +214,7 @@ namespace Beagle.Daemon {
 		{
 			SqliteConnection c;			
 			c = new SqliteConnection ();
-			c.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
-				+ ",encoding=UTF-8,URI=file:" + GetDbPath (directory);
+			c.ConnectionString = "version=3,encoding=UTF-8,URI=file:" + GetDbPath (directory);
 			c.Open ();
 			return c;
 		}
@@ -241,44 +267,21 @@ namespace Beagle.Daemon {
 			if (path != null && path != "/" && path.EndsWith ("/"))
 				path = path.TrimEnd ('/');
 
-			SqliteCommand command;
-			SqliteDataReader reader;
-
 			if (! GetPathFlag (path))
 				return null;
 
 			FileAttributes attr = null;
-			bool found_too_many = false;
 
 			// We need to quote any 's that appear in the strings
 			// (int particular, in the path)
 			string directory = FileSystem.GetDirectoryNameRootOk (path).Replace ("'", "''");
 			string filename = Path.GetFileName (path).Replace ("'", "''");
 			lock (connection) {
-				command = SqliteUtils.QueryCommand (connection,
-								    "directory='{0}' AND filename='{1}'",
-								    directory, filename);
-				reader = SqliteUtils.ExecuteReaderOrWait (command);
-				
-				if (SqliteUtils.ReadOrWait (reader)) {
-					attr = GetFromReader (reader);
-					
+				ReadCommand.Parameters.AddWithValue ("@dir",directory);
+				ReadCommand.Parameters.AddWithValue ("@fname",filename);
+				using (SqliteDataReader reader = SqliteUtils.ExecuteReaderOrWait (ReadCommand)) {
 					if (SqliteUtils.ReadOrWait (reader))
-						found_too_many = true;
-				}
-				reader.Close ();
-				command.Dispose ();
-
-				// If we found more than one matching record for a given
-				// directory and filename, something has gone wrong.
-				// Since we have no way of knowing which one is correct
-				// and which isn't, we delete them all and return
-				// null.  (Which in most cases will force a re-index.
-				if (found_too_many) {
-					SqliteUtils.DoNonQuery (connection,
-								"DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
-								directory, filename);
-					attr = null;
+						attr = GetFromReader (reader);
 				}
 			}
 
@@ -302,17 +305,19 @@ namespace Beagle.Daemon {
 				if (filter_name == null)
 					filter_name = "";
 				filter_name = filter_name.Replace ("'", "''");
-
-				ret = SqliteUtils.DoNonQuery (connection,
-							"INSERT OR REPLACE INTO file_attributes " +
-							" (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
-							" VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
-							GuidFu.ToShortString (fa.UniqueId),
-							fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
-							StringFu.DateTimeToString (fa.LastWriteTime),
-							StringFu.DateTimeToString (fa.LastAttrTime),
-							filter_name,
-							fa.FilterVersion);
+				string[] param= new string [] { "@unique_id", "@directory", "@filename", "@last_mtime", "@last_attrtime", "@filter_name", "@filter_version"};
+				object[] vals =	new object [] {
+								GuidFu.ToShortString (fa.UniqueId),
+								fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
+								StringFu.DateTimeToString (fa.LastWriteTime),
+								StringFu.DateTimeToString (fa.LastAttrTime),
+								filter_name,
+								fa.FilterVersion};
+				for (int i=0; i < param.Length; i++)
+					InsertCommand.Parameters.AddWithValue (param[i], vals[i]);
+				
+				ret = SqliteUtils.DoNonQuery (InsertCommand);
+				
 			}
 
 			return (ret != 0);
@@ -335,10 +340,9 @@ namespace Beagle.Daemon {
 
 				// If a transaction has been requested, start it now.
 				MaybeStartTransaction ();
-
-				SqliteUtils.DoNonQuery (connection,
-							"DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
-							directory, filename);
+				DeleteCommand.Parameters.AddWithValue ("@directory", directory);
+				DeleteCommand.Parameters.AddWithValue ("@filename", filename);
+				SqliteUtils.DoNonQuery (DeleteCommand);
 			}
 		}
 
