@@ -492,6 +492,15 @@ namespace Beagle.Daemon {
 			private bool strip_extra_property_info = false;
 			private bool tokenize_email_hostname = false;
 
+			private NoiseEmailHostFilter.LinkCallback add_link = null;
+
+			public NoiseEmailHostFilter.LinkCallback AddLink {
+				set {
+				    lock (this)
+					    add_link = value;
+				}
+			}
+
 			public BeagleAnalyzer (bool is_indexing_analyzer)
 			{
 				if (is_indexing_analyzer) {
@@ -538,15 +547,25 @@ namespace Beagle.Daemon {
 					}
 				} else if (fieldName == "PropertyKeyword")
 					return new LowerCaseFilter (new SingletonTokenStream (reader.ReadToEnd ()));
+				else if (fieldName == "Properties")
+					return new WhitespaceTokenizer (new StringReader (reader.ReadToEnd ()));
+				else if (fieldName == "TextLinks")
+					return new WhitespaceTokenizer (new StringReader (reader.ReadToEnd ()));
 
 				TokenStream outstream;
 				outstream = base.TokenStream (fieldName, reader);
 
+				NoiseEmailHostFilter.LinkCallback add_link_callback = null;
+				lock (this) {
+					if (fieldName == "Text")
+						add_link_callback = add_link;
+				}
+
 				if (fieldName == "Text"
 				    || fieldName == "HotText"
 				    || fieldName == "PropertyText"
 				    || is_text_prop) {
-					outstream = new NoiseEmailHostFilter (outstream, tokenize_email_hostname);
+					outstream = new NoiseEmailHostFilter (outstream, tokenize_email_hostname, add_link_callback);
 					// Sharing Stemmer is not thread safe.
 					// Currently our underlying lucene indexing is not done in multiple threads.
 					StemmerInfo stemmer_info = GetStemmer (DEFAULT_STEMMER_LANGUAGE);
@@ -557,11 +576,11 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		static private Analyzer indexing_analyzer = new BeagleAnalyzer (true);
-		static private Analyzer query_analyzer = new BeagleAnalyzer (false);
+		static private BeagleAnalyzer indexing_analyzer = new BeagleAnalyzer (true);
+		static private BeagleAnalyzer query_analyzer = new BeagleAnalyzer (false);
 
-		static protected Analyzer IndexingAnalyzer { get { return indexing_analyzer; } }
-		static protected Analyzer QueryAnalyzer { get { return query_analyzer; } }
+		static protected BeagleAnalyzer IndexingAnalyzer { get { return indexing_analyzer; } }
+		static protected BeagleAnalyzer QueryAnalyzer { get { return query_analyzer; } }
 
 		////////////////////////////////////////////////////////////////
 
@@ -615,6 +634,8 @@ namespace Beagle.Daemon {
 		// Exposing this is a little bit suspicious.
 		static protected string PropertyToFieldName (PropertyType type, string key)
 		{
+			if (type == PropertyType.Internal)
+				return key;
 			return String.Format ("prop:{0}:{1}", TypeToCode (type), key);
 
 		}
@@ -871,8 +892,63 @@ namespace Beagle.Daemon {
 					
 				AddPropertyToDocument (prop, target_doc);
 			}
+#if ENABLE_RDF_ADAPTER
+
+			// Now add the whitespace separated list of links extracted from the document of the text
+			// Add the property to the primary document. Why primary ?
+			// Because it stays with the "Text" property
+			Fieldable links_field = new LinksField (indexable.Links);
+			primary_doc.Add (links_field);
+
+			// Finally add a field containing a whitespace separated list of other fields in the document
+			AddFieldProperies (primary_doc);
+			if (secondary_doc != null)
+				AddFieldProperies (secondary_doc);
+#endif
 		}
 
+#if ENABLE_RDF_ADAPTER
+		private class LinksField : Fieldable {
+			IList<string> links;
+			internal LinksField (IList<string> links)
+			{
+				this.links = links;
+			}
+
+			public void  SetBoost(float boost) { }
+			public float GetBoost() { return 1.0f; }
+			public System.String Name() { return "TextLinks"; }
+
+			public System.String StringValue()
+			{
+				if (links == null)
+					return String.Empty;
+
+				StringBuilder sb = new StringBuilder ();
+				foreach (string link in links) {
+					sb.Append (link);
+					sb.Append (" ");
+				}
+
+				return sb.ToString ();
+			}
+
+			public System.IO.TextReader ReaderValue() { return null; }
+			public byte[] BinaryValue() { return null; }
+			public bool IsStored() { return false; }
+			public bool IsIndexed() { return true; }
+			public bool IsTokenized() { return true; }
+			public bool IsCompressed() { return false; }
+			public bool IsTermVectorStored() { return false; }
+			public bool IsStoreOffsetWithTermVector() { return false; }
+			public bool IsStorePositionWithTermVector() { return false; }
+			public bool IsBinary() { return false; }
+			public bool GetOmitNorms() { return true; }
+			public void  SetOmitNorms(bool omitNorms) { }
+			public bool IsLazy() { return false; }
+		}
+#endif
+
 		static private Document CreateSecondaryDocument (Uri uri, Uri parent_uri)
 		{
 			Document secondary_doc = new Document ();
@@ -943,6 +1019,9 @@ namespace Beagle.Daemon {
 				}
 			}
 
+#if ENABLE_RDF_ADAPTER
+			AddFieldProperies (new_doc);
+#endif
 			return new_doc;
 		}
 
@@ -964,9 +1043,40 @@ namespace Beagle.Daemon {
 				}
 			}
 
+#if ENABLE_RDF_ADAPTER
+			AddFieldProperies (doc);
+#endif
 			return doc;
 		}
 
+		// Add a new field with whitespace separated names of the existing fields
+		static protected void AddFieldProperies (Document doc)
+		{
+			const string Separator = " ";
+
+			StringBuilder sb = new StringBuilder ();
+			bool seen_properties = false;
+
+			foreach (Fieldable f in doc.Fields ()) {
+				if (f.Name () == "Properties") {
+					seen_properties = true;
+					continue;
+				}
+
+				sb.Append (f.Name ());
+				sb.Append (Separator);
+			}
+
+			if (sb.Length > 0)
+				sb.Length -= Separator.Length;
+
+			if (seen_properties)
+				doc.RemoveFields ("Properties");
+
+			Field field = new Field ("Properties", sb.ToString (), Field.Store.NO, Field.Index.TOKENIZED);
+			doc.Add (field);
+		}
+
 		static protected Uri GetUriFromDocument (Document doc)
 		{
 			string uri;
@@ -997,8 +1107,8 @@ namespace Beagle.Daemon {
 
 		static protected void AddPropertiesToHit (Hit hit, Document doc, bool from_primary_index)
 		{
+			Property prop;
 			foreach (Field f in doc.Fields ()) {
-				Property prop;
 				prop = GetPropertyFromDocument (f, doc, from_primary_index);
 				if (prop != null)
 					hit.AddProperty (prop);
@@ -1093,7 +1203,7 @@ namespace Beagle.Daemon {
 
 		public static bool IsStopWord (string stemmed_word)
 		{
-			return ArrayFu.IndexOfString (StopAnalyzer.ENGLISH_STOP_WORDS, stemmed_word) != -1;
+			return Array.IndexOf<string> (StopAnalyzer.ENGLISH_STOP_WORDS, stemmed_word) != -1;
 		}
 
 		//////////////////////////////////////////////////////////////////////////////
@@ -1455,7 +1565,9 @@ namespace Beagle.Daemon {
 			// This gives a chance to modify create new queries based on
 			// backend specific properties
 
-			abstract_part = query_part_hook (abstract_part);
+			if (query_part_hook != null)
+				abstract_part = query_part_hook (abstract_part);
+
 			if (abstract_part == null)
 				return;
 
@@ -1671,11 +1783,13 @@ namespace Beagle.Daemon {
 				else
 					field_name = PropertyToFieldName (part.Type, part.Key);
 
+				// Details of the conversion here depends on BeagleAnalyzer::TokenStream
 				if (part.Type == PropertyType.Text)
 					primary_query = StringToQuery (field_name, part.Value, term_list);
 				else {
 					Term term;
-					if (field_name.StartsWith ("prop:k:" + Property.PrivateNamespace))
+					// FIXME: Handle date queries for other date fields
+					if (part.Type == PropertyType.Internal || field_name.StartsWith ("prop:k:" + Property.PrivateNamespace))
 						term = new Term (field_name, part.Value);
 					else
 						term = new Term (field_name, part.Value.ToLower ());
@@ -1824,6 +1938,7 @@ namespace Beagle.Daemon {
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
+					rav.Refcount ++; // add one refcount to keep this reader open until...
 
 					directory_rav_map [directory] = rav;
 					reader_rav_map [reader] = rav;
@@ -1832,8 +1947,11 @@ namespace Beagle.Daemon {
 				}
 
 				version = IndexReader.GetCurrentVersion (directory);
-				
+
 				if (version != rav.Version) {
+					// unref once and then close reader or call UnrefReaderAndVersion_Unlocked as a single step
+					UnrefReaderAndVersion_Unlocked (rav);
+
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
@@ -1871,6 +1989,21 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		// Special function to permanently remove this reader
+		// Called when StaticQueryables need to unload
+		static internal void CloseReader (IndexReader reader)
+		{
+			lock (reader_rav_map) {
+				ReaderAndVersion rav = (ReaderAndVersion) reader_rav_map [reader];
+
+				if (rav != null) {
+					rav.Refcount --; // Remove one extra refcount that was added to keep the reader opened
+					UnrefReaderAndVersion_Unlocked (rav);
+				} else
+					reader.Close ();
+			}
+		}
+
 		static public void ReleaseSearcher (LNS.IndexSearcher searcher)
 		{
 			IndexReader reader = searcher.GetIndexReader ();
@@ -2041,6 +2174,11 @@ namespace Beagle.Daemon {
 		// For a large index, this will be very slow and will consume
 		// a lot of memory.  Don't call it without a good reason!
 		public ICollection GetHitsForUris (ICollection uris)
+		{
+			return GetHitsForUris (uris, null);
+		}
+
+		public ICollection GetHitsForUris (ICollection uris, FieldSelector fields)
 		{
 			Hashtable hits_by_uri = UriFu.NewHashtable ();
 
@@ -2052,7 +2190,9 @@ namespace Beagle.Daemon {
 			LNS.Hits primary_hits = primary_searcher.Search (uri_query);
 
 			for (int i = 0; i < primary_hits.Length (); i++) {
-				Document doc = primary_hits.Doc (i);
+				Document doc = ((fields == null) ?
+					primary_hits.Doc (i) :
+					primary_hits.Doc (i, fields));
 
 				Uri u = GetUriFromDocument (doc);
 
@@ -2064,7 +2204,9 @@ namespace Beagle.Daemon {
 				LNS.Hits secondary_hits = secondary_searcher.Search (uri_query);
 
 				for (int i = 0; i < secondary_hits.Length (); i++) {
-					Document doc = secondary_hits.Doc (i);
+					Document doc = ((fields == null) ?
+						secondary_hits.Doc (i) :
+						secondary_hits.Doc (i, fields));
 
 					Uri uri = GetUriFromDocument (doc);
 					Hit hit = (Hit) hits_by_uri [uri];
@@ -2078,5 +2220,31 @@ namespace Beagle.Daemon {
 
 			return hits_by_uri.Values;
 		}
+
+		//////////////////////////////////////////////////
+
+		public static void DebugHook ()
+		{
+			lock (reader_rav_map) {
+				Lucene.Net.Store.Directory dir;
+				ReaderAndVersion rav;
+				IndexReader reader;
+
+				Log.Debug ("Cached readers per directory:");
+				foreach (DictionaryEntry entry in directory_rav_map) {
+					dir = entry.Key as Lucene.Net.Store.Directory;
+					rav = entry.Value as ReaderAndVersion;
+					Log.Debug ("\tDirectory {0} has {1} readers open", dir.GetLockID (), rav.Refcount);
+				}
+
+				Log.Debug ("Opened readers:");
+				foreach (DictionaryEntry entry in reader_rav_map) {
+					reader = entry.Key as IndexReader;
+					rav = entry.Value as ReaderAndVersion;
+					dir = reader.Directory ();
+					Log.Debug ("\t{2} readers v{0} opened for directory {1}", rav.Version, dir.GetLockID (), rav.Refcount);
+				}
+			}
+		}
 	}
 }
