@@ -622,8 +622,21 @@ namespace Beagle.Daemon {
 			Document primary_doc = null, secondary_doc = null;
 
 			try {
+#if ENABLE_RDF_ADAPTER
+				// Add a callback to extract emails and links from the anaylyzer
+				// and add them to secondary_doc's "References" field.
+				IndexingAnalyzer.AddLink = delegate (string s, bool email)
+							    {
+								    // Only add emails for now
+								    // NoiseFilter is not good with URLs
+								    if (! email || indexable.Links == null)
+									    return;
+								    indexable.Links.Add (s);
+							    };
+#endif
 				BuildDocuments (indexable, out primary_doc, out secondary_doc);
 				primary_writer.AddDocument (primary_doc);
+				IndexingAnalyzer.AddLink = null;
 			} catch (Exception ex) {
 					
 				// If an exception was thrown, something bad probably happened
@@ -652,6 +665,12 @@ namespace Beagle.Daemon {
 				secondary_writer.AddDocument (secondary_doc);
 			}
 
+#if ENABLE_RDF_ADAPTER
+			// Store the extracted links in the textcache
+			if (! disable_textcache && text_cache != null)
+				text_cache.AddLinks (indexable.Uri, indexable.Links);
+#endif
+
 			AdjustItemCount (1);
 		}
 
